@@ -5,15 +5,15 @@ use {
     jsonrpc_ipc_server::{RequestContext, ServerBuilder},
     jsonrpc_server_utils::tokio,
     log::*,
-    serde::{Deserialize, Serialize},
-    solana_core::validator::ValidatorStartProgress,
-    solana_gossip::{cluster_info::ClusterInfo, contact_info::ContactInfo},
+    solana_core::{
+        consensus::Tower, tower_storage::TowerStorage, validator::ValidatorStartProgress,
+    },
+    solana_gossip::cluster_info::ClusterInfo,
     solana_sdk::{
         exit::Exit,
         signature::{read_keypair_file, Keypair, Signer},
     },
     std::{
-        fmt::{self, Display},
         net::SocketAddr,
         path::{Path, PathBuf},
         sync::{Arc, RwLock},
@@ -30,79 +30,10 @@ pub struct AdminRpcRequestMetadata {
     pub validator_exit: Arc<RwLock<Exit>>,
     pub authorized_voter_keypairs: Arc<RwLock<Vec<Arc<Keypair>>>>,
     pub cluster_info: Arc<RwLock<Option<Arc<ClusterInfo>>>>,
+    pub tower_storage: Arc<dyn TowerStorage>,
 }
 impl Metadata for AdminRpcRequestMetadata {}
 
-#[derive(Debug, Deserialize, Serialize)]
-pub struct AdminRpcContactInfo {
-    pub id: String,
-    pub gossip: SocketAddr,
-    pub tvu: SocketAddr,
-    pub tvu_forwards: SocketAddr,
-    pub repair: SocketAddr,
-    pub tpu: SocketAddr,
-    pub tpu_forwards: SocketAddr,
-    pub tpu_vote: SocketAddr,
-    pub rpc: SocketAddr,
-    pub rpc_pubsub: SocketAddr,
-    pub serve_repair: SocketAddr,
-    pub last_updated_timestamp: u64,
-    pub shred_version: u16,
-}
-
-impl From<ContactInfo> for AdminRpcContactInfo {
-    fn from(contact_info: ContactInfo) -> Self {
-        let ContactInfo {
-            id,
-            gossip,
-            tvu,
-            tvu_forwards,
-            repair,
-            tpu,
-            tpu_forwards,
-            tpu_vote,
-            rpc,
-            rpc_pubsub,
-            serve_repair,
-            wallclock,
-            shred_version,
-        } = contact_info;
-        Self {
-            id: id.to_string(),
-            last_updated_timestamp: wallclock,
-            gossip,
-            tvu,
-            tvu_forwards,
-            repair,
-            tpu,
-            tpu_forwards,
-            tpu_vote,
-            rpc,
-            rpc_pubsub,
-            serve_repair,
-            shred_version,
-        }
-    }
-}
-
-impl Display for AdminRpcContactInfo {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        writeln!(f, "Identity: {}", self.id)?;
-        writeln!(f, "Gossip: {}", self.gossip)?;
-        writeln!(f, "TVU: {}", self.tvu)?;
-        writeln!(f, "TVU Forwards: {}", self.tvu_forwards)?;
-        writeln!(f, "Repair: {}", self.repair)?;
-        writeln!(f, "TPU: {}", self.tpu)?;
-        writeln!(f, "TPU Forwards: {}", self.tpu_forwards)?;
-        writeln!(f, "TPU Votes: {}", self.tpu_vote)?;
-        writeln!(f, "RPC: {}", self.rpc)?;
-        writeln!(f, "RPC Pubsub: {}", self.rpc_pubsub)?;
-        writeln!(f, "Serve Repair: {}", self.serve_repair)?;
-        writeln!(f, "Last Updated Timestamp: {}", self.last_updated_timestamp)?;
-        writeln!(f, "Shred Version: {}", self.shred_version)
-    }
-}
-
 #[rpc]
 pub trait AdminRpc {
     type Metadata;
@@ -128,8 +59,8 @@ pub trait AdminRpc {
     #[rpc(meta, name = "removeAllAuthorizedVoters")]
     fn remove_all_authorized_voters(&self, meta: Self::Metadata) -> Result<()>;
 
-    #[rpc(meta, name = "contactInfo")]
-    fn contact_info(&self, meta: Self::Metadata) -> Result<AdminRpcContactInfo>;
+    #[rpc(meta, name = "setIdentity")]
+    fn set_identity(&self, meta: Self::Metadata, keypair_file: String) -> Result<()>;
 }
 
 pub struct AdminRpcImpl;
@@ -206,9 +137,30 @@ impl AdminRpc for AdminRpcImpl {
         Ok(())
     }
 
-    fn contact_info(&self, meta: Self::Metadata) -> Result<AdminRpcContactInfo> {
+    fn set_identity(&self, meta: Self::Metadata, keypair_file: String) -> Result<()> {
+        debug!("set_identity request received");
+
+        let identity_keypair = read_keypair_file(&keypair_file).map_err(|err| {
+            jsonrpc_core::error::Error::invalid_params(format!(
+                "Failed to read identity keypair from {}: {}",
+                keypair_file, err
+            ))
+        })?;
+
+        // Ensure a Tower exists for the new identity and exit gracefully.
+        // ReplayStage will be less forgiving if it fails to load the new tower.
+        Tower::restore(meta.tower_storage.as_ref(), &identity_keypair.pubkey()).map_err(|err| {
+            jsonrpc_core::error::Error::invalid_params(format!(
+                "Unable to load tower file for new identity: {}",
+                err
+            ))
+        })?;
+
         if let Some(cluster_info) = meta.cluster_info.read().unwrap().as_ref() {
-            Ok(cluster_info.my_contact_info().into())
+            solana_metrics::set_host_id(identity_keypair.pubkey().to_string());
+            cluster_info.set_keypair(Arc::new(identity_keypair));
+            warn!("Identity set to {}", cluster_info.id());
+            Ok(())
         } else {
             Err(jsonrpc_core::error::Error::invalid_params(
                 "Retry once validator start up is complete",
